# frozen_string_literal: true

# Return Hermes::HousingRequest attributes from parsed xml

module Hermes
  module BoClient
    module Mappings
      class HousingRequestReverseMap   
        include Hermes::BoClient::MappingHelper
             
        def initialize(xml)
          @xml = xml
        end

        def check_if_true_or_nil_or_false(param, reversed= false)
          if(param == 'true')
            return reversed ? false : true
          elsif param == "false"
            return reversed ? true : false
          else
            return nil
          end
        end
        def build_housing_request
 
          demande_logement = @xml["interfaceNuu"]["demande"]["demandeLogement"] #["personnePhysique"]["demandeur"]["mel"]
          personnePhysique = demande_logement["personnePhysique"]
          demandeur = personnePhysique["demandeur"]
          return {} unless personnePhysique['situation']

          attributes = {
            cerfa_version: "4",
            ec_law_ressources: demande_logement["ressourcesLoiEC"],
            lessor_siren: personnePhysique["situation"]&.fetch("compSIRENOrgBailleur", nil),
            electronical_renewal: demande_logement["renouvellementElec"],
            widen_request: demande_logement["logementRecherche"]["demandeElargie"],
            lessor_name: personnePhysique['situation']['compNomOrgBailleur'],
            work_contract_type: [
              demandeur["situationProfessionnelle"]["typeContratTravail"]&.fetch("code", nil)
            ],
            dalo: ['ACTIF', 'ACTREL'].include?(@xml["interfaceNuu"]["demande"]&.fetch("informationDalo", nil)&.fetch("statutDalo", nil)&.fetch("code", nil)),
            with_car_park: check_if_true_or_nil_or_false(demande_logement["logementRecherche"]["avecParking"]),
            mutation: !!demande_logement['renouvellement'],
            seeking_first_floor: check_if_true_or_nil_or_false(demande_logement["logementRecherche"]["refusRezDeChaussee"], true),
            seeking_elevator: check_if_true_or_nil_or_false(demande_logement["logementRecherche"]["refusSansAscenseur"], true),#possible souci avec ce champ
            seeked_max_rent: demande_logement["logementRecherche"]["montantLoyer"].to_i,
            precisions: demande_logement["precision"],
            seeked_housing_categories: [
              demande_logement["logementRecherche"]["categorieLogementRecherche"]&.fetch("code", nil)
            ],
            is_house_owner: personnePhysique["situation"]["proprietaire"],
            expulsion_judgement_date: personnePhysique["situation"]&.fetch("compDateJugement", nil),
            requester_firstname: demandeur["prenoms"],
            requester_lastname: demandeur["nom"]|| demandeur["nomPatronymique"],
            requester_fullname: demandeur["prenoms"] + " " + (demandeur["nom"]|| demandeur["nomPatronymique"]),
            requester_email: demandeur["mel"],
            # created_at: demande_logement["dateCreationDemande"],
            sne_created_at: demande_logement["dateCreationDemande"],
            last_sne_renewal_date: demande_logement['renouvellement']&.fetch("dateRenouvellement"),
            radiation_date: demande_logement["radiation"]&.fetch("dateRadiation", nil),
            radiation_motive: demande_logement["radiation"]&.fetch("motifRadiation", nil)&.fetch("code", nil),
            #10095 ajout de nouveaux champs cerfa v4
            have_monthly_income: demande_logement['ressourcesMensuelles'],
            disabled_person_number: demande_logement['informationHandicap']&.fetch("nombreHandicap", 0) || 0,
            #10106 ajout de nouveaux champs cerfa v4
            owner_info_needed: demande_logement["logementRecherche"]["infoProprietaire"]
          }

          attributes[:forcing_duplicate] = (demande_logement["forcerDoublon"] != "false") if demande_logement["forcerDoublon"]
          
          attributes[:actual_housing_details] = {}
          situation = personnePhysique["situation"] || {}
          if situation
            modeleLogement = situation["listeModeleLogement"]&.fetch("modeleLogement", nil)
            attributes[:housing_type_list] = modeleLogement.is_a?(Array) ? [] : [housing_type_list_v4_map(modeleLogement&.fetch("code", nil))]
            if modeleLogement.is_a?(Array)
              attributes[:housing_type_list] = []
              modeleLogement.each_with_index do |val|
                attributes[:housing_type_list].push (housing_type_list_v4_map(val&.fetch("code", nil)))
              end
            end
          end
      
     
      
          if personnePhysique["personneANaitre"]
            attributes[:unborn_children] = [{"anticipated_birth" => personnePhysique["personneANaitre"]["dateNaissancePrevue"], "nb_child_birth" => personnePhysique["personneANaitre"]["nombre"]}]
          else
            attributes[:unborn_children] = []
          end
         
          detailSituation = situation["listeSituation"] ? situation["listeSituation"]["detailSituation"] : []
          if (detailSituation.is_a? Hash)
            attributes[:motivations] = [situation_motivation_v4_map(detailSituation&.fetch("code", nil))]
          else
            attributes[:motivations] = []
            detailSituation&.each do |situ|
              attributes[:motivations].push(situation_motivation_v4_map(situ.fetch("code", nil)))
            end
            
          end
          attributes[:motivations].flatten!
          if(demandeur["situationProfessionnelle"]&.fetch("typeContratTravail")&.fetch("code", nil)== "ASS")
            attributes[:motivations].push("26")
          else
            xml_coapps = demande_logement["personnePhysique"]&.fetch("listeCodemandeur")&.fetch("codemandeur")
            if(xml_coapps)
              xml_coapps = xml_coapps.is_a?(Array) ? xml_coapps : [xml_coapps]
              done= false
              xml_coapps.each_with_index do |xml_coapp, i|
              
                if(!done && xml_coapp["situationProfessionnelle"]&.fetch("typeContratTravail")&.fetch("code", nil)== "ASS")
                  attributes[:motivations].push("26")
                  done= true
                end
              end
            end
          end
          
          if(demande_logement["anru"]== "true")
            attributes[:motivations].push("27")
          end
          attributes[:seeked_types] = []
          typeLogement = demande_logement["logementRecherche"]["listeNombrePieces"]["nombrePiecesRecherchees"]
          typeLogement = transform_in_array(typeLogement)
       
          typeLogement&.each do |type|
            attributes[:seeked_types].push(wanted_housing_type_v4_map(type&.fetch("code", nil).to_s))
          end
       
          attributes[:dependents] = []
          demandeur["listeRessourceRecuePersACharge"] = transform_in_array(demandeur["listeRessourceRecuePersACharge"])
          demandeur["listeRessourceRecuePersACharge"]&.each_with_index do |persACharge, i|
            
            if persACharge && persACharge["detailRessource"]
              attributes[:dependents][i] ||= {}
              attributes[:dependents][i]["monthly_incomes"] ||= {}
              ressources = transform_in_array(persACharge["detailRessource"])
              ressources.each do |ressource|
                attributes[:dependents][i]["monthly_incomes"][ressource&.fetch("code", nil).to_s] = ressource["montant"].to_i
              end
            
            end
          end
     
          demandeur["listeRessourceVerseePersACharge"] = transform_in_array(demandeur["listeRessourceVerseePersACharge"])
          demandeur["listeRessourceVerseePersACharge"]&.each_with_index do |persACharge, i|
            if persACharge && persACharge["detailRessource"]
              attributes[:dependents][i] ||= {}
              attributes[:dependents][i]["monthly_charges"] ||= {}
              attributes[:dependents][i]["monthly_charges"][persACharge["detailRessource"]["ressource"]&.fetch("code", nil).to_s] = persACharge["detailRessource"]["montant"].to_i
            end
          end
          if personnePhysique["listePersonneACharge"] && personnePhysique["listePersonneACharge"]["personneACharge"]
            personnePhysique["listePersonneACharge"]["personneACharge"] = transform_in_array(personnePhysique["listePersonneACharge"]["personneACharge"])
            personnePhysique["listePersonneACharge"]["personneACharge"]&.each_with_index do |persACharge, i|
              if persACharge
                
                attributes[:dependents][i] ||= {}
                attributes[:dependents][i]["last_name"] = persACharge["nom"]
                attributes[:dependents][i]["first_name"] = persACharge["prenom"]
                attributes[:dependents][i]["birth_date"] = persACharge["dateNaissance"]
                attributes[:dependents][i]["rel_type"] = persACharge["lienParente"]&.fetch("code", nil)
                attributes[:dependents][i]["sex"] = persACharge["sexe"]&.fetch("code", nil)
          
              end
            end
          end
          dependents_index = attributes[:dependents].length 
          if (personnePhysique["listePersonneEnGarde"])
            personnePhysique["listePersonneEnGarde"]["personneEnGarde"] = transform_in_array(personnePhysique["listePersonneEnGarde"]["personneEnGarde"])
            personnePhysique["listePersonneEnGarde"]["personneEnGarde"]&.each_with_index do |persACharge, i|
              if persACharge
                attributes[:dependents][i + dependents_index] ||= {}
                attributes[:dependents][i + dependents_index]["last_name"] ||= persACharge["nom"]
                attributes[:dependents][i + dependents_index]["first_name"] ||= persACharge["prenom"]
                attributes[:dependents][i + dependents_index]["birth_date"] ||= persACharge["dateNaissance"]
                attributes[:dependents][i + dependents_index]["coparentality"] = persACharge&.fetch("coParentalite")&.fetch("code", nil)
                attributes[:dependents][i + dependents_index]["sex"] ||= persACharge["sexe"]&.fetch("code", nil)
              end
            end
            
          end
      
          attributes[:seeked_localisation_list] = []
          localisationSouhaite = demande_logement["logementRecherche"]["listeLocalisationSouhaite"]["localisationSouhaite"]
          localisationSouhaite = transform_in_array(localisationSouhaite)
          localisationSouhaite.each_with_index do |loc, i|
            if (loc)
              res = {}
              res["code"] = loc["commune"]["code"]
              res["codesPostaux"] = loc["codePostal"].present? ? [loc["codePostal"]] : []
              res["seeked_epci_first_quartil"] = loc["seuilEPCIPremierQuartile"]
              res["seeked_quartil_belongings"] = loc["quartileAppartenance"]
              res["nom"]=nil
              attributes[:seeked_localisation_list].push res
            end
          end
          attributes[:disability_details] = []

          if demande_logement["listeHandicap"]
            if demande_logement["listeHandicap"].is_a?(Hash)
              demande_logement["listeHandicap"] = transform_in_array(demande_logement["listeHandicap"]["handicap"])
            end
            attributes[:disability_details] = [{}]
            demande_logement["listeHandicap"].each_with_index do |handicap, i|
              
              handicap = handicap
              
              attributes[:disability_details][i] ||= {}
              attributes[:disability_details][i] = {"disability_states": handicap["etatHandicap"]&.fetch("code", nil),
                                                    "walking_capacities": handicap["capaciteMarcheHandicap"]&.fetch("code", nil),
              
              
                                                    "birth_date": handicap["dateNaissance"],
                                                    "accompanied": handicap["accompagne"] == "true",
                                                  
                                                    "other_disability": handicap["handicapAutre"] == "true",
                                                    "special_need": handicap["besoinParticulier"],
                                                    "other_need": handicap["besoinAutre"],
              }
              if handicap["referent"]
                attributes[:disability_details][i]["disability_referent"] = {
                  "recognized_disability": (handicap["referent"]["handicapReconnu"] == "true"),
                  "near_home": (handicap["referent"]["procheDomicile"] == "true"),
                  "name": handicap["referent"]["nom"],
                  "phone": handicap["referent"]["tel"],
                  "email": handicap["referent"]["mel"],
                  "address_insee_code": handicap["referent"]["commune"]&.fetch("code", nil)
                }

                if handicap["referent"]["voie"] && handicap["referent"]["codePostal"] #&& handicap["referent"]["commune"]&.fetch("code", nil)
                  attributes[:disability_details][i]["disability_referent"]["address"] = (handicap["referent"]["numero"]||"") + ", " + (handicap["referent"]["voie"]||"")  + ", " + (handicap["referent"]["codePostal"]||"") # + ", " + (handicap["referent"]["commune"]&.fetch("code", nil)||"") 
                end 
              end
              if handicap["listeNatureHandicapSensoriel"]
                attributes[:disability_details][i]["sensory_disability_kinds"] = extract_codes(handicap, 'listeNatureHandicapSensoriel')
                attributes[:disability_details][i]["sensory_disability"]= true
              end
              if handicap["listeNatureHandicapMoteur"]
                attributes[:disability_details][i]["physical_disability_kinds"] = extract_codes(handicap, 'listeNatureHandicapMoteur')
                attributes[:disability_details][i]["physical_disability"]= true
              end
              if handicap["listeAideTechniqueHandicap"]
                attributes[:disability_details][i]["disability_technical_helps"] = extract_codes(handicap, 'listeAideTechniqueHandicap')
              end
              if handicap["listeEquipementHandicap"]
                attributes[:disability_details][i]["disability_equipments"] = extract_codes(handicap, 'listeEquipementHandicap')
              end
            end
          end

          attributes
        end

        def housing_type_list_v4_map(code)
          map = {
            "LLS"=>"HLM",
            "LSLLP"=>"LP",
            "HLF"=>"RSLL",
            "REU"=>"RE",
            "LOGT"=>"SLHT",
            "HHSH"=>"HSH",
            "CDEFCM"=>"HCDFCM",
            "CPE"=>"HCPE",
            "CP"=>"HCP",
            "SAHFB"=>"SD",
            "HOST"=>"OST",
            "HCAM"=>"CAMCAR",
            "HH"=>"HHOT",
            "HHM"=>"HABMOB"
          }
          return map[code] || code
        end
        def wanted_housing_type_v4_map(code)
          map = {
            "P1"=>"T1",
            "C"=>"C",
            "P2"=>"T2",
            "P3"=>"T3",
            "P4"=>"T4",
            "P5"=>"T5",
            "P6P"=>"T6P"
          }
          return map[code] || code
        end
        def owned_housing_type_v4_map(code)
          map = {
            "P1"=>"T1",
            "P2"=>"T2",
            "P3"=>"T3",
            "P4"=>"T4",
            "P5"=>"T5",
            "P6P"=>"T6P"
          }
          return map[code] || code
        end

        def situation_motivation_v4_map(code)
        
          map = {
            "LBD" => "02", 
            "LIN" => "25",
            "LND" => "24",
            "LRP" => "04",
            "EPE" => "05",
            "LHA" => ["07","08"],
            "TTC" => "09",
            "TTG" => "10",
            "FDS" => "11",
            "FDP" => "12",
            "TTP" => "13",
            "FRF" => ["14","15"],
            "TCH" => "28",
            "LET" => "19",
            "LES" => "20",
            "LEF" => "21",
            "FVC" => "06",

          }
          return map[code] || code
        end
      
        def extract_codes(handicap, attribute_name)
          codes =[]
          return [] unless handicap[attribute_name]
          if handicap[attribute_name].values.is_a? (Array)
       
            if handicap[attribute_name].values[0].is_a?(Array)
              handicap[attribute_name].values[0].each do |h|
                codes.push(h&.fetch("code", nil))
              end
            else
              codes.push (handicap[attribute_name].values[0]&.fetch("code", nil))
            end
          else
            codes.push( handicap[attribute_name].values&.fetch("code", nil))
          end
          return codes
        end 
      end
    end
  end
end
